/*
BSL-Scripter-3.0.1.0

==ExecutionClass.cpp==

Copyright (c) 2008  2015 Texas Instruments Incorporated

All rights reserved not granted herein.

Limited License.

Texas Instruments Incorporated grants a world-wide, royalty-free,
non-exclusive license under copyrights and patents it now or
hereafter owns or controls to make, have made, use, import,
offer to sell and sell ("Utilize") this software subject to
the terms herein.  With respect to the foregoing patent license,
such license is granted  solely to the extent that any such patent
is necessary to Utilize the software alone.  The patent license
shall not apply to any combinations which include this software,
other than combinations with devices manufactured by or for TI
(TI Devices). No hardware patent is licensed hereunder.

Redistributions must preserve existing copyright notices and
reproduce this license (including the above copyright notice
and the disclaimer and (if applicable) source code license
limitations below) in the documentation and/or other materials
provided with the distribution

Redistribution and use in binary form, without modification,
are permitted provided that the following conditions are met:

*	No reverse engineering, decompilation, or disassembly of
this software is permitted with respect to any software
provided in binary form.

*	any redistribution and use are licensed by TI for use only
with TI Devices.

*	Nothing shall obligate TI to provide you with source code
for the software licensed and provided to you in object code.

If software source code is provided to you, modification and
redistribution of the source code are permitted provided that
the following conditions are met:

*	any redistribution and use of the source code, including any
resulting derivative works, are licensed by TI for use only
with TI Devices.

*	any redistribution and use of any object code compiled from
the source code and any resulting derivative works, are licensed
by TI for use only with TI Devices.

Neither the name of Texas Instruments Incorporated nor the names
of its suppliers may be used to endorse or promote products derived
from this software without specific prior written permission.

DISCLAIMER.

THIS SOFTWARE IS PROVIDED BY TI AND TIS LICENSORS "AS IS" AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL TI AND TIS LICENSORS
BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

Version:	Release:		Author:
3.0.1.0		Aug/07/2015		Fatmawati Santosa

*/

#include "stdafx.h"

/*******************************************************************************
*Class:       ExecutionCommand
*Function:    setTarget
*Description: interface to the selected device family
*Parameters:  ActualCommandParams actualCommandParams:      get family device
*Returns:     none
*******************************************************************************/
void ExecutionCommand::setTarget(ActualCommandParams actualCommandParams)
{
	if (actualCommandParams.family == Family::P4xx)
	{
		target.reset(new P432);
	}
	else if (actualCommandParams.family == Family::ROM)
	{
		target.reset(new Rom);
	}
	else
	{
		target.reset(new FramFlash);
	}

}

/*******************************************************************************
*Class:       ExecutionCommand
*Function:    callFunc
*Description: interface to the selected BSL command
*Parameters:  ActualCommandParams actualCommandParams:      get the command
*Returns:     none
*******************************************************************************/
bool ExecutionCommand::retRunFunc()
{
	return runFunc;
}

/*******************************************************************************
*Class:       ExecutionCommand
*Function:    callFunc
*Description: interface to the selected BSL command
*Parameters:  ActualCommandParams actualCommandParams:      get the command
*Returns:     none
*******************************************************************************/
void ExecutionCommand::callFunc(ActualCommandParams actualCommandParams)
{
	try
	{
		target->readParameters(actualCommandParams);
	}
	catch (const char* e)//(std::runtime_error& e)
	{
		std::cout << e << std::endl;
		runFunc = false;
		//std::cerr << e.what() << std::endl;
		return;
	}

	runFunc = true;

	try
	{
		switch (actualCommandParams.command)
		{
			
			case (Command::CHANGE_BAUD_RATE) : dataBuffer = target->changeBaudRate(); break;
			case (Command::CRC_CHECK_32) : dataBuffer = target->crcCheck32(); break;
			case (Command::CRC_CHECK) : dataBuffer = target->crcCheck(); break;
			case (Command::DELAY) :	 target->delay();	break;
			case (Command::ERASE_CHECK) : dataBuffer = target->eraseCheck(); break;
			case (Command::ERASE_MAIN_OR_INFO) : dataBuffer = target->eraseMainOrInfo(); break;
			case (Command::ERASE_SEGMENT_32) : dataBuffer = target->eraseSegment32(); break;
			case (Command::ERASE_SEGMENT) : dataBuffer = target->eraseSegment(); break;
			case (Command::SET_PC_32) : dataBuffer = target->setPc32(); break;
			case (Command::SET_PC) : dataBuffer = target->setPc(); break;
			case (Command::MASS_ERASE) : dataBuffer = target->massErase(); break;
			case (Command::RX_DATA_BLOCK_FAST) : dataBuffer = target->rxDataBlockFast(); break;
			case (Command::RX_DATA_BLOCK_32) : dataBuffer = target->rxDataBlock32(); break;
			case (Command::RX_DATA_BLOCK) : dataBuffer = target->rxDataBlock(); break;
			case (Command::RX_PASSWORD_32) : dataBuffer = target->rxPassword32(); break;
			case (Command::RX_PASSWORD) : dataBuffer = target->rxPassword(); break;
			case (Command::SET_MEMORY_OFFSET) : dataBuffer = target->setMemoryOffset(); break;
			case (Command::TOGGLE_INFO) : dataBuffer = target->toggleInfo(); break;
			case (Command::TX_DATA_BLOCK_32) : dataBuffer = target->txDataBlock32();	break;
			case (Command::TX_DATA_BLOCK) : dataBuffer = target->txDataBlock();	break;
			case (Command::TX_BSL_VERSION_32) : dataBuffer = target->bslVersion32(); break;
			case (Command::TX_BSL_VERSION) : dataBuffer = target->bslVersion(); break;
			case (Command::TX_BUFFER_SIZE) : dataBuffer = target->txBufferSize();	break;
			case (Command::VERBOSE) : target->verbose();	break;

			default:break;	
		}

	}
	catch (std::runtime_error& e)
	{
		std::cerr << e.what() << std::endl;
	}

}

/*******************************************************************************
*Class:       ExecutionCommand
*Function:    retDataBuffer
*Description: return DataBuffer that contains BSL commands
*Parameters:  none
*Returns:     DataBuffer: data of BSL commands to be sent
*******************************************************************************/
DataBuffer ExecutionCommand::retDataBuffer()
{
	return dataBuffer;
}

/*******************************************************************************
*Class:       ExecutionCommand
*Function:    retNumResponseBytes
*Description: return number of response bytes of respective command
*Parameters:  none
*Returns:     uint32_t:	number of response byte
*******************************************************************************/
uint32_t ExecutionCommand::retNumResponseBytes()
{
	uint32_t num = target->retNumResponseBytes();
	return num;
}

/*******************************************************************************
*Class:       ExecutionCommand
*Function:    retNewBaudRate
*Description: return the set up baud rate
*Parameters:  none
*Returns:     uint32_t
*******************************************************************************/
uint32_t ExecutionCommand::retNewBaudRate()
{
	uint32_t num = target->retNewBaudRate();
	return num;
}

/*******************************************************************************
*Class:       ExecutionCommand
*Function:    retNewBaudRate
*Description: return the set up baud rate
*Parameters:  none
*Returns:     uint32_t
*******************************************************************************/
void ExecutionCommand::setDataBlock(DataBlock dataBlock)
{
	target->setDataBlock(dataBlock);
}

/*******************************************************************************
*Class:       ExecutionCommand
*Function:    initVerbose
*Description: turn off the verbose mode for the beginning
*Parameters:  none
*Returns:     none
*******************************************************************************/
void ExecutionCommand::initVerbose()
{
	target->initVerbose();
}

/*******************************************************************************
*Class:       ExecutionCommand
*Function:    retVerbose
*Description: return the Verbose status from the user
*Parameters:  none
*Returns:     bool true : Verbose is on
*                  false: Verbose is off
*******************************************************************************/
bool ExecutionCommand::retVerbose()
{
	return target->retVerbose();
}

/*******************************************************************************
*Class:       ExecutionCommunication
*Function:    setTarget
*Description: interface to the selected protocol communication
*Parameters:  ActualCommandParams actualCommandParams:	get selected protocol
*Returns:     none
*******************************************************************************/
void ExecutionCommunication::setTarget(ActualCommandParams actualCommandParams)
{
	if (actualCommandParams.protocol == Protocol::USB)
	{
		targetComm.reset(new UsbComm);
	}
	else
	{
		targetComm.reset(new UartComm(2));
	}
}

/*******************************************************************************
*Class:       ExecutionCommunication
*Function:    init
*Description: initiate the open port for communication
*Parameters:  ActualCommandParams actualCommandParams:	get selected protocol
*Returns:     none
*******************************************************************************/
void ExecutionCommunication::init(ActualCommandParams actualCommandParams)
{
	targetComm->init(actualCommandParams);
}

/*******************************************************************************
*Class:       ExecutionCommunication
*Function:    transmitBufferInit432
*Description: initiate the invoke sequence for P432 family
*Parameters:  none
*Returns:     none
*******************************************************************************/
void ExecutionCommunication::transmitBufferInit432()
{
	targetComm->transmitBufferInit432();
}

/*******************************************************************************
*Class:       ExecutionCommunication
*Function:    transmitBuffer
*Description: performs transmission BSL command to the device
*Parameters:  DataBuffer  dataBuffer:  contains the BSL commands
*             bool        verbose:     status of verbose
*Returns:     none
*******************************************************************************/
void ExecutionCommunication::transmitBuffer(DataBuffer dataBuffer, bool verbose)
{
	targetComm->transmitBuffer(dataBuffer, verbose);
}

/*******************************************************************************
*Class:       ExecutionCommunication
*Function:    receiveBuffer
*Description: performs receive BSL response from the device
*Parameters:  uint32_t  size:        number of bytes that will be received
*             bool      verbose:     status of verbose
*Returns:     uint8_t*:				 pointer to the receive buffer
*******************************************************************************/
uint8_t* ExecutionCommunication::receiveBuffer(uint32_t size, bool verbose)
{
	uint8_t *ptr = targetComm->receiveBuffer(size, verbose);
	return ptr;
}

/*******************************************************************************
*Class:       ExecutionCommunication
*Function:    close
*Description: close the communication port
*Parameters:  none
*Returns:     none
*******************************************************************************/
void ExecutionCommunication::close()
{
	targetComm->close();
}

/*******************************************************************************
*Class:       ExecutionCommunication
*Function:    changeBaudRate
*Description: perfoms the baud rate change value
*Parameters:  uint32_t baudRate:	new baud rate value
*Returns:     none
*******************************************************************************/
void ExecutionCommunication::changeBaudRate(uint32_t baudRate)
{
	targetComm->changeBaudRate(baudRate);
}

/*******************************************************************************
*Class:       ExecutionCommunication
*Function:    retBytesWritten
*Description: return the number of bytes that been written to the output stream
*Parameters:  none
*Returns:     uint32_t baudRate:	number of written bytes
*******************************************************************************/
uint32_t ExecutionCommunication::retBytesWritten()
{
	uint32_t num = targetComm->retBytesWritten();
	return num;
}

/*******************************************************************************
*Class:       ExecutionCommunication
*Function:    retBytesRead
*Description: return the number of bytes that been read from the output stream
*Parameters:  none
*Returns:     uint32_t baudRate:	number of written bytes
*******************************************************************************/
uint32_t ExecutionCommunication::retBytesRead()
{
	uint32_t num = targetComm->retBytesRead();
	return num;
}

/*******************************************************************************
*Class:       ExecutionCommunication
*Function:    resetBytesWritten
*Description: reset the number of bytes that been written to the output stream
*Parameters:  none
*Returns:     none
*******************************************************************************/
void ExecutionCommunication::resetBytesWritten()
{
	targetComm->resetBytesWritten();
}

/*******************************************************************************
*Class:       ExecutionCommunication
*Function:    resetBytesRead
*Description: reset the number of bytes that been read from the output stream
*Parameters:  none
*Returns:     none
*******************************************************************************/
void ExecutionCommunication::resetBytesRead()
{
	targetComm->resetBytesWritten();
}


