#pragma once

#include "stdafx.h"

class P432 : public InterfaceTarget
{
	void readParameters(ActualCommandParams actualCommandParams) override;

	DataBuffer changeBaudRate() override;
	DataBuffer crcCheck32() override;
	DataBuffer crcCheck() override;
	void       delay() override;
	DataBuffer eraseCheck() override;
	DataBuffer eraseMainOrInfo() override;
	DataBuffer eraseSegment32() override;
	DataBuffer eraseSegment() override;
	DataBuffer setPc32() override;
	DataBuffer setPc() override;
	DataBuffer massErase() override;
	DataBuffer rebootReset() override;
	DataBuffer rxDataBlockFast() override;
	DataBuffer rxDataBlock32() override;
	DataBuffer rxDataBlock() override;
	DataBuffer rxPassword32() override;
	DataBuffer rxPassword() override;
	DataBuffer setMemoryOffset() override;
	DataBuffer toggleInfo() override;
	DataBuffer txDataBlock32() override;
	DataBuffer txDataBlock() override;
	DataBuffer bslVersion32() override;
	DataBuffer bslVersion() override;
	DataBuffer txBufferSize() override;
	void       initVerbose() override;
	void       verbose() override;
	bool       retVerbose() override;

	uint32_t   retNewBaudRate() override;
	uint32_t   retNumResponseBytes() override;
	void       setDataBlock(DataBlock dataBlockStream) override;

private:

	//Header byte BSL commands
	const uint8_t RX_DATA_BLOCK = 0x10;
	const uint8_t RX_PASSWORD = 0x11;
	const uint8_t ERASE_SEGMENT = 0x12;
	const uint8_t TOGGLE_INFO = 0x13;
	const uint8_t ERASE_BLOCK = 0x14;
	const uint8_t MASS_ERASE = 0x15;
	const uint8_t CRC_CHECK = 0x16;
	const uint8_t SET_PC = 0x17;
	const uint8_t TX_DATA_BLOCK = 0x18;
	const uint8_t TX_BSL_VERSION = 0x19;
	const uint8_t TX_BUFFER_SIZE = 0x1A;
	const uint8_t RX_DATA_BLOCK_FAST = 0x1B;

	//Header byte BSL commands 32-bit
	const uint8_t RX_DATA_BLOCK_32 = 0x20;
	const uint8_t RX_PASSWORD_32 = 0x21;
	const uint8_t ERASE_SEGMENT_32 = 0x22;
	const uint8_t REBOOT_RESET = 0x25;
	const uint8_t CRC_CHECK_32 = 0x26;
	const uint8_t SET_PC_32 = 0x27;
	const uint8_t TX_DATA_BLOCK_32 = 0x28;
	const uint8_t TX_BSL_VERSION_32 = 0x19;

	//Header byte PI
	const uint8_t CHANGE_BAUD_RATE = 0x52;
	const uint8_t BAUD430_4800 = 0x01;
	const uint8_t BAUD430_9600 = 0x02;
	const uint8_t BAUD430_19200 = 0x03;
	const uint8_t BAUD430_38400 = 0x04;
	const uint8_t BAUD430_57600 = 0x05;
	const uint8_t BAUD430_115200 = 0x06;


	//Return response byte
	uint32_t numResponseBytes;

	//read parameters
	uint32_t address;
	uint32_t address32;
	uint32_t length;
	uint32_t newBaudRate;
	uint8_t commandBaudRate;
	uint32_t delayMs;

	uint16_t numPass;
	const uint8_t PASS_16 = 16;
	const uint8_t PASS_32 = 32;

	uint8_t selPass;
	const uint8_t PASS_DEFAULT = 1;
	const uint8_t PASS_FILE = 2;

	bool verboseToggle;

	uint32_t totalBytesReceived;

	DataBlock dataBlock;

};