/*
BSL-Scripter-3.0.1.0


Copyright (c) 2008  2015 Texas Instruments Incorporated

All rights reserved not granted herein.

Limited License.

Texas Instruments Incorporated grants a world-wide, royalty-free,
non-exclusive license under copyrights and patents it now or
hereafter owns or controls to make, have made, use, import,
offer to sell and sell ("Utilize") this software subject to
the terms herein.  With respect to the foregoing patent license,
such license is granted  solely to the extent that any such patent
is necessary to Utilize the software alone.  The patent license
shall not apply to any combinations which include this software,
other than combinations with devices manufactured by or for TI
(TI Devices). No hardware patent is licensed hereunder.

Redistributions must preserve existing copyright notices and
reproduce this license (including the above copyright notice
and the disclaimer and (if applicable) source code license
limitations below) in the documentation and/or other materials
provided with the distribution

Redistribution and use in binary form, without modification,
are permitted provided that the following conditions are met:

*	No reverse engineering, decompilation, or disassembly of
this software is permitted with respect to any software
provided in binary form.

*	any redistribution and use are licensed by TI for use only
with TI Devices.

*	Nothing shall obligate TI to provide you with source code
for the software licensed and provided to you in object code.

If software source code is provided to you, modification and
redistribution of the source code are permitted provided that
the following conditions are met:

*	any redistribution and use of the source code, including any
resulting derivative works, are licensed by TI for use only
with TI Devices.

*	any redistribution and use of any object code compiled from
the source code and any resulting derivative works, are licensed
by TI for use only with TI Devices.

Neither the name of Texas Instruments Incorporated nor the names
of its suppliers may be used to endorse or promote products derived
from this software without specific prior written permission.

DISCLAIMER.

THIS SOFTWARE IS PROVIDED BY TI AND TIS LICENSORS "AS IS" AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL TI AND TIS LICENSORS
BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

Version:	Release:		Author:
3.0.1.0		Aug/07/2015		Fatmawati Santosa

*/

#include "stdafx.h"

//Constructor
ScripterStream::ScripterStream()
{
	for (uint16_t i = 0; i < MAX_BUFFER_SIZE; i++)
	{
		dataBlockRead.data[i] = 0;
	}
	dataBlockRead.startAddress = 0;
	dataBlockRead.numOfBytes = 0;

	fileScriptName = "";
	line = "";
}

/*******************************************************************************
*Class:       ScripterStream
*Function:    getThePathFile
*Description: set the path for the respective file
*Parameters:  none
*Returns:     std::string resultPath: complete streaming file
*******************************************************************************/
std::string ScripterStream::getThePathFile(std::string stInput, std::string relativePath)
{
	std::string resultPath = "";
 
#if defined(_WIN32) || defined(_WIN64)
	const char* sepSymbol = "\\";
	const char* sepSymbol_ = "/";
	uint8_t pos_doubleDots = stInput.find_first_of(":");
	bool goAbsolute = false;
	if (pos_doubleDots == 1)
	{
		goAbsolute = true;
	}
#else
	const char* sepSymbol = "/";
	uint8_t pos_doubleDots = stInput.find_first_of("/");
	bool goAbsolute = false;
	if (pos_doubleDots == 0)
	{
		goAbsolute = true;
	}
#endif

	uint8_t pos_upFolder = stInput.find_first_of("..");

	if (goAbsolute)//if (pos_doubleDots == 1)  // ABSOLUTE PATH
	{
		resultPath = stInput;
	}
	else // UP_FOLDER
	{
		boost::char_separator<char> sep("\\,/");
		boost::tokenizer<boost::char_separator<char>> tokensRelative(relativePath, sep);

		uint8_t cntDeepRelativePath = 0;
		std::string tempBufString[10];

		for (const string& t : tokensRelative)
		{
			tempBufString[cntDeepRelativePath] = t;
			cntDeepRelativePath++;
		}

		if (pos_upFolder == 0)   //UP_FOLDER
		{
			boost::tokenizer<boost::char_separator<char>> tokens(stInput, sep);

			uint8_t cntUpFolder = 0;
			std::string strTemp = "";
			for (const string& t : tokens)
			{
				if (t == "..")
				{
					cntUpFolder++;
				}
				else
				{
					strTemp = t;
					break;
				}
			}
			
			for (uint8_t i = 0; i < (cntDeepRelativePath - cntUpFolder - 1); i++)
			{
				resultPath += tempBufString[i] + sepSymbol;
			}
			resultPath += strTemp;
		}
		else // DOWN FOLDER
		{
			for (uint8_t i = 0; i < (cntDeepRelativePath - 1); i++)
			{
				resultPath += tempBufString[i] + sepSymbol;
			}
			resultPath += stInput;
		}
  	}
#if defined(_WIN32) || defined(_WIN64)
	return resultPath;
#else
	std::string temp = "/" + resultPath;
	return temp;
#endif
}


/*******************************************************************************
*Class:       ScripterStream
*Function:    startScripter
*Description: open the script file and get the path directory
*Parameters:  none
*Returns:     std::string retPathDir: Path directory of executable file
*******************************************************************************/
std::string ScripterStream::startScripter(char * ch)
{
	std::cout << "---------------------------------------------------------" << std::endl;
	std::cout << "BSL Scripter 3.0.1.0" << std::endl;
	std::cout << "---------------------------------------------------------" << std::endl;

	if (ch == nullptr)
	{
		std::cout << "Enter the name of file script : ";
		std::cin >> fileName;
	}
	else
	{
		fileName = (reinterpret_cast<char*>(ch));
	}

	boost::filesystem::path fullPath(boost::filesystem::current_path());
	
	std::string pathDir = fullPath.string();
	pathDir += "/addition";

	std::string newPath = getThePathFile(fileName, pathDir);

	std::string showFileName = newPath;
	std::replace(showFileName.begin(), showFileName.end(), '\\', '/');
  	std::cout << "Input file script is : " << showFileName << std::endl;

	fileScriptStream.open(newPath, ios::in);

	if (!fileScriptStream.good())
	{
		throw std::runtime_error("File Scripter is not available...");
	}

	return newPath;
}

/*******************************************************************************
*Class:       ScripterStream
*Function:    getLine
*Description: check if command still available or end of file is reached
*Parameters:  none
*Returns:     bool true: line command available
*             bool false: end of file reach
*******************************************************************************/
bool ScripterStream::getLine()
{
	if (fileScriptStream.eof())
	{
		return false;
	}
	else
	{
		std::getline(fileScriptStream, line);
		return true;
	}
}

/*******************************************************************************
*Class:       ScripterStream
*Function:    retStringCommand
*Description: return string line from the script file
*Parameters:  none
*Returns:     std::string line: command 
*******************************************************************************/
std::string ScripterStream::retStringCommand()
{
	return line;
}

/*******************************************************************************
*Class:       ScripterStream
*Function:    retDataBlockRead
*Description: read and copy data from streaming input 
*Parameters:  none
*Returns:     DataBlock dataBlockRead: data to be transmitted to BSL device
*******************************************************************************/
DataBlock ScripterStream::retDataBlockRead()
{
	return dataBlockRead;
}

/*******************************************************************************
*Class:       ScripterStream
*Function:    onGoingReadStatus
*Description: assign read status from streaming file
*Parameters:  none
*Returns:     none
*******************************************************************************/
void ScripterStream::onGoingReadStatus()
{
	readStatus = READ_ON_GOING;
}

/*******************************************************************************
*Class:       ScripterStream
*Function:    finishReadStatus
*Description: assign finish read status from streaming file
*Parameters:  none
*Returns:     none
*******************************************************************************/
void ScripterStream::finishReadStatus()
{
	readStatus = READ_DONE;
}

/*******************************************************************************
*Class:       ScripterStream
*Function:    retReadStatus
*Description: return read status
*Parameters:  none
*Returns:     uint8_t readStatus: READ_ON_GOING
*                                 READ_DONE
*******************************************************************************/
uint8_t ScripterStream::retReadStatus()
{
	return readStatus;
}

/*******************************************************************************
*Class:       ScripterStream
*Function:    setSendStatus
*Description: assign set send status
*Parameters:  none
*Returns:     none
*******************************************************************************/
void ScripterStream::setSendStatus()
{
	sendStatus = SEND_DATA_BLOCK;
}

/*******************************************************************************
*Class:       ScripterStream
*Function:    setSendStatus
*Description: assign clear send status
*Parameters:  none
*Returns:     none
*******************************************************************************/
void ScripterStream::clearSendStatus()
{
	sendStatus = KEEP_DATA_BLOCK;
}

/*******************************************************************************
*Class:       ScripterStream
*Function:    retSendStatus
*Description: assign clear send status
*Parameters:  none
*Returns:     uint8_t retSendStatus: SEND_DATA_BLOCK
*                                    KEEP_DATA_BLOCK
*******************************************************************************/
uint8_t ScripterStream::retSendStatus()
{
	return sendStatus;
}

/*******************************************************************************
*Class:       ScripterStream
*Function:    retSendStatus
*Description: initiate the pointer to streaming input file
*Parameters:  ActualCommandParams actualCommandParam: get the input file
*             std::string mainPathDir: path directory of exe and input file
*Returns:     none
*             exception if the file does not exist
*******************************************************************************/
void ScripterStream::initReadTxtFile(ActualCommandParams actualCommandParams, std::string mainPathDir)
{
	if ((actualCommandParams.family == Family::P4xx) && (actualCommandParams.command == Command::RX_PASSWORD_32)
		&& (actualCommandParams.paramsInput[1] != ""))
	{
		numBytesToRead = 256;
	}
	else
	{
		numBytesToRead = actualCommandParams.maxBufferSize;
	}
	
	//fileName = mainPathDir + actualCommandParams.paramsInput[1];
	fileName = getThePathFile(actualCommandParams.paramsInput[1], mainPathDir);
	std::string showFileName = fileName;
	std::replace(showFileName.begin(), showFileName.end(), '\\', '/');
	std::cout << "Read Txt File : " << showFileName << std::endl;
	bytesRead = 0;
	onGoingReadStatus();
	clearSendStatus();
	changeAddress = NO_CHANGE_ADDRESS;
	fileReadStream.open(fileName, ios::in);

	if (!fileReadStream.good())
	{
		throw std::runtime_error("File is not available/accessed by other program...");
	}
}

/*******************************************************************************
*Class:       ScripterStream
*Function:    closeReadTxtFile
*Description: close the streaming input file
*Parameters:  none
*Returns:     none
*******************************************************************************/
void ScripterStream::closeReadTxtFile()
{
	fileReadStream.close();
}

/*******************************************************************************
*Class:       ScripterStream
*Function:    readTxtFile
*Description: close the streaming input file
*Parameters:  ActualCommandParams actualCommandParams: get the 
*Returns:     none
*******************************************************************************/
void ScripterStream::readTxtFile()
{
//#define DEBUG_BLOCK_USB

	std::string   tempLine;
	std::string   tempToken;

	if (bytesRead < numBytesToRead)
	{
		if (!fileReadStream.eof())
		{
			std::getline(fileReadStream, tempLine);

			boost::char_separator<char> sep("\t\r ");
			boost::tokenizer<boost::char_separator<char>> tokens(tempLine, sep);

			for (const string& t : tokens) 
			{
				tempToken = t;
				uint8_t pos = tempToken.find_first_of("@");
				if (pos == 0)
				{
					tempToken.erase(0, 1);
					saveLastReadAddress = std::stoul(tempToken, nullptr, 16);
					changeAddress = CHANGE_ADDRESS;

					if (bytesRead != 0)
					{
						bytesRead = 0;
						setSendStatus();

						//Debugging usb
#ifdef DEBUG_BLOCK_USB
						std::cout << "SETUP DATA BLOCK USB : in change address " << std::endl;
						std::cout << "start address : " << intToHexAddr(dataBlockRead.startAddress) << std::endl;
						for (uint32_t i = 0; i < dataBlockRead.numOfBytes; i++)
						{
							std::cout << "[" << intToHex(dataBlockRead.data[i]) << "] ";
						}
						std::cout << "------------------------------" << std::endl;
						std::cout << "------------------------------" << std::endl;
#endif
						return;
					}


				}
				else
				{
					if ((tempToken == "q") || (tempToken == "Q"))
					{
						dataBlockRead.numOfBytes = bytesRead;
						setSendStatus();
						finishReadStatus();
						fileReadStream.close();

						//Debugging usb
#ifdef DEBUG_BLOCK_USB						
						std::cout << "SETUP DATA BLOCK USB : in change address " << std::endl;
						std::cout << "start address : " << intToHexAddr(dataBlockRead.startAddress) << std::endl;
						for (uint32_t i = 0; i < dataBlockRead.numOfBytes; i++)
						{
							std::cout << "[" << intToHex(dataBlockRead.data[i]) << "] ";
						}
						std::cout << "------------------------------" << std::endl;
						std::cout << "------------------------------" << std::endl;
#endif
						return;
					}
					else
					{
						onGoingReadStatus();

						if (changeAddress == CHANGE_ADDRESS)//if ((changeAddress == CHANGE_ADDRESS) && (bytesRead != 0))
						{
							clearSendStatus();
							dataBlockRead.startAddress = saveLastReadAddress;
							changeAddress = NO_CHANGE_ADDRESS;
						}


						dataBlockRead.data[bytesRead] = (uint8_t)(std::stoul(tempToken, nullptr, 16));
						bytesRead++;

						if (bytesRead == numBytesToRead) 
						{

							dataBlockRead.numOfBytes = bytesRead;
							
							saveLastReadAddress = dataBlockRead.numOfBytes + dataBlockRead.startAddress;
							changeAddress = CHANGE_ADDRESS;
							bytesRead = 0;
							setSendStatus();

#ifdef DEBUG_BLOCK_USB							
							std::cout << "SETUP DATA BLOCK USB : because buffer is full " << std::endl;
							std::cout << "start address : " << intToHexAddr(dataBlockRead.startAddress) << std::endl;
						
							for (uint32_t i = 0; i < dataBlockRead.numOfBytes; i++)
							{
								std::cout << "[" << intToHex(dataBlockRead.data[i]) << "] ";
							}
							std::cout << "------------------------------" << std::endl;
							std::cout << "------------------------------" << std::endl;
#endif	
							return;
						}
					}
				}
			} 
		}
	}
}
 
/*******************************************************************************
*Class:       ScripterStream
*Function:    intToHex
*Description: convert the uint8_t type into hex in string type
*             need to cast the uint8_t to uint32_t
*Parameters:  uint8_t input: integer input to be converted to hex
*Returns:     std::string  stream.str(): the string format of hex value
*******************************************************************************/
std::string ScripterStream::intToHex(uint8_t input)
{
	std::stringstream stream;
	stream << std::setfill('0')
		<< std::setw(sizeof(input) * 2)
		<< std::hex
		<< (uint32_t)input;
	return stream.str();
}

/*******************************************************************************
*Class:       ScripterStream
*Function:    intToHexAddr
*Description: convert the uint32_t type into hex in string type
*Parameters:  uint8_t input: integer input to be converted to hex
*Returns:     std::string  stream.str(): the string format of hex value
*******************************************************************************/
std::string ScripterStream::intToHexAddr(uint32_t address)
{
	std::stringstream stream;
	stream << std::setfill('0')
		<< std::hex
		<< address;
	return stream.str();
}

/*******************************************************************************
*Class:       ScripterStream
*Function:    intToHexAddr
*Description: convert the uint32_t type into hex in string type
*Parameters:  uint8_t input: integer input to be converted to hex
*Returns:     std::string  stream.str(): the string format of hex value
*******************************************************************************/
void ScripterStream::initWriteTxtFile(ActualCommandParams actualCommandParams, std::string mainPathDir)
{
	//fileName = mainPathDir + actualCommandParams.paramsInput[3];
	fileName = getThePathFile(actualCommandParams.paramsInput[3], mainPathDir);
	std::string showFileName = fileName;
	std::replace(showFileName.begin(), showFileName.end(), '\\', '/');
	std::cout << "Write Txt File : " << showFileName << std::endl;
	
	fileWriteStream.open(fileName, ios::out);
	actualCommandParams.paramsInput[1].erase(0, 2);
	fileWriteStream << "@" << actualCommandParams.paramsInput[1] << "\n";
	
	if (actualCommandParams.protocol == Protocol::USB)
	{
		maxBufSize  = 64;
		startHeader1 = 3;
		startHeader2 = 3;
		minTempSize = 4;
	}
	else
	{
		maxBufSize = 266;
		startHeader1 = 5;
		startHeader2 = 4;
		minTempSize = 2;
	}

	if (!fileWriteStream.good())
	{
		throw std::runtime_error("File to write the file could not be created...");
	}
}

/*******************************************************************************
*Class:       ScripterStream
*Function:    writeTxtFile
*Description: write the received data from BSL device to streaming output file
*Parameters:  uint8_t* buf: pointer to the buffer that contains received data
*             uint32_t size: number of bytes that will be written to file
*Returns:     std::string  stream.str(): the string format of hex value
*******************************************************************************/
void ScripterStream::writeTxtFile(uint8_t *buf, uint32_t size)
{
	uint32_t k=0;
	uint32_t switchLoop=0;
	uint32_t byteWritten = 0;

	uint32_t tempSize = size - minTempSize;

	for (uint32_t i = 0; i < tempSize; i++, buf++)
	{
		if (((switchLoop == 0) && ((k >= startHeader1) && (k < (maxBufSize - 2)))) || //switchLoop 1 condition
			((switchLoop == 1) && ((k >= startHeader2) && (k < (maxBufSize - 2))))    //switchLoop 2 condition
		   )	  
		{
				std::string tempString = intToHex(*buf);
				fileWriteStream << tempString << " ";
				byteWritten++;

				if (byteWritten  % 16 == 0)
				{
					fileWriteStream << "\n";
				}
		}

		k++;
		if (k == maxBufSize)
		{
			k = 0;
			switchLoop = 1;
		}
	}
 	fileWriteStream << "q";
}


void ScripterStream::closeWriteTxtFile()
{
	fileWriteStream.close();
}