/*
BSL-Scripter-3.0.1.0

==UartComm.cpp==

Copyright (c) 2008  2015 Texas Instruments Incorporated

All rights reserved not granted herein.

Limited License.

Texas Instruments Incorporated grants a world-wide, royalty-free, 
non-exclusive license under copyrights and patents it now or 
hereafter owns or controls to make, have made, use, import, 
offer to sell and sell ("Utilize") this software subject to 
the terms herein.  With respect to the foregoing patent license, 
such license is granted  solely to the extent that any such patent 
is necessary to Utilize the software alone.  The patent license 
shall not apply to any combinations which include this software, 
other than combinations with devices manufactured by or for TI 
(TI Devices). No hardware patent is licensed hereunder.

Redistributions must preserve existing copyright notices and
reproduce this license (including the above copyright notice
and the disclaimer and (if applicable) source code license
limitations below) in the documentation and/or other materials
provided with the distribution

Redistribution and use in binary form, without modification,
are permitted provided that the following conditions are met:

*	No reverse engineering, decompilation, or disassembly of
this software is permitted with respect to any software
provided in binary form.

*	any redistribution and use are licensed by TI for use only
with TI Devices.

*	Nothing shall obligate TI to provide you with source code
for the software licensed and provided to you in object code.

If software source code is provided to you, modification and
redistribution of the source code are permitted provided that
the following conditions are met:

*	any redistribution and use of the source code, including any
resulting derivative works, are licensed by TI for use only
with TI Devices.

*	any redistribution and use of any object code compiled from
the source code and any resulting derivative works, are licensed
by TI for use only with TI Devices.

Neither the name of Texas Instruments Incorporated nor the names
of its suppliers may be used to endorse or promote products derived
from this software without specific prior written permission.

DISCLAIMER.

THIS SOFTWARE IS PROVIDED BY TI AND TIS LICENSORS "AS IS" AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL TI AND TIS LICENSORS
BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

Version:	Release:		Author:
3.0.1.0		Aug/07/2015		Fatmawati Santosa

*/

#include "stdafx.h"

#if defined(_WIN32) || defined(_WIN64)

#else
	#include <unistd.h>     // UNIX standard function definitions
	#include <fcntl.h>      // File control definitions
	#include <termios.h>    // POSIX terminal control definitions
	#include <sys/ioctl.h>

	struct termios2 {
		tcflag_t c_iflag;
		tcflag_t c_oflag;
		tcflag_t c_cflag;
		tcflag_t c_lflag;
		cc_t c_line;
		cc_t c_cc[19];
		speed_t c_ispeed;
		speed_t c_ospeed;
	};
#endif
 
//Constructor
UartComm::UartComm(uint32_t timeout) :	port(io_service),
										timer(port.get_io_service()),
										readError(true)
{

}

/*******************************************************************************
*Class:       UartComm
*Function:    invokeBSL (not tested yet)
*Description: invoke the BSL with the test reset sequence
*Parameters:  none
*Returns:     none
*******************************************************************************/
void UartComm::invokeBSL()
{
	port.set_option(RESETControl(1));
	port.set_option(TESTControl(0));
	std::this_thread::sleep_for(std::chrono::milliseconds(100));
	
	port.set_option(TESTControl(1));
	std::this_thread::sleep_for(std::chrono::microseconds(10));

	port.set_option(TESTControl(0));
	std::this_thread::sleep_for(std::chrono::microseconds(20));

	port.set_option(TESTControl(1));
	std::this_thread::sleep_for(std::chrono::microseconds(10));

	port.set_option(RESETControl(0));
	std::this_thread::sleep_for(std::chrono::microseconds(10));

	port.set_option(TESTControl(0));
	std::this_thread::sleep_for(std::chrono::microseconds(10));

	std::this_thread::sleep_for(std::chrono::milliseconds(100));
}

/*******************************************************************************
*Class:       UartComm
*Function:    init
*Description: setup the uart communication port
*Parameters:  ActualCommandParams actualCommandParams: get the commport
*                                                      and baud rate value
*Returns:     none
*******************************************************************************/
void UartComm::init(ActualCommandParams actualCommandParams)
{
	const char * c = actualCommandParams.commPort.c_str();

	port.open(c);

	const boost::asio::serial_port_base::flow_control FLOW(boost::asio::serial_port_base::flow_control::none);
	const boost::asio::serial_port_base::parity PARITY(boost::asio::serial_port_base::parity::none);
	//const boost::asio::serial_port_base::parity PARITY(boost::asio::serial_port_base::parity::even);
	const boost::asio::serial_port_base::stop_bits STOP(boost::asio::serial_port_base::stop_bits::one);

	port.set_option(boost::asio::serial_port_base::character_size(8));
	port.set_option(FLOW);
	port.set_option(PARITY);
	port.set_option(STOP);

	invokeBSL();

	uint32_t initBaudRate;
	if ((actualCommandParams.protocol == Protocol::UART) || 
		(actualCommandParams.protocol == Protocol::USB) ||
		(actualCommandParams.protocol == Protocol::I2C)
	   )
	{
		initBaudRate = actualCommandParams.baudRate + 1;
	}
  
#if defined(_WIN32) || defined(_WIN64)
	
	port.set_option(boost::asio::serial_port::baud_rate(initBaudRate));

#elif defined(__APPLE__)

	int fd = port.native_handle();
	ioctl (fd, _IOW('T', 2, speed_t), &initBaudRate, 1);

#else

	struct termios2 tio;
	int fd = port.native_handle();
	ioctl(fd, TCGETS2, &tio);
	tio.c_cflag &= ~CBAUD;
	tio.c_cflag |= 0010000;
	tio.c_ispeed = initBaudRate;
	tio.c_ospeed = initBaudRate;
	ioctl(fd, TCSETS2, &tio);

#endif

	std::this_thread::sleep_for(std::chrono::milliseconds(1000));

#if defined(_WIN32) || defined(_WIN64)

	port.set_option(boost::asio::serial_port::baud_rate(actualCommandParams.baudRate));

#elif defined(__APPLE__)

	ioctl(fd, _IOW('T', 2, speed_t), &actualCommandParams.baudRate, 1);

#else

	ioctl(fd, TCGETS2, &tio);
	tio.c_cflag &= ~CBAUD;
	tio.c_cflag |= 0010000;
	tio.c_ispeed = actualCommandParams.baudRate;
	tio.c_ospeed = actualCommandParams.baudRate;
	ioctl(fd, TCSETS2, &tio);

#endif

	std::this_thread::sleep_for(std::chrono::milliseconds(1000));
}

/*******************************************************************************
*Class:       UartComm
*Function:    transmitBufferInit432
*Description: send the init sequence for P432 family
*Parameters:  none
*Returns:     none
*******************************************************************************/
void UartComm::transmitBufferInit432()
{
	uint8_t uartTxBuffer[1] = { 0xFF };

	boost::system::error_code ec;

	boost::asio::write(port, boost::asio::buffer(uartTxBuffer, 1), ec);

	if (ec)
	{
		throw runtime_error("Error from writing occurs...");
	}
}

/*******************************************************************************
*Class:       UartComm
*Function:    transmitBuffer
*Description: send data buffer of BSl command
*Parameters:  DataBuffer dataBuffer: buffer contain BSL command to be transmitted
              bool verbose: set or unset the verbose mode
*Returns:     none
*******************************************************************************/
void UartComm::transmitBuffer(DataBuffer dataBuffer, bool verbose)
{
	cleanBigBuffer();

	uint8_t uartTxBuffer[257 + 5];

	uartTxBuffer[0] = 0x80; //Header byte
	uartTxBuffer[1] = (uint8_t)(dataBuffer.size & 0xFF);
	uartTxBuffer[2] = (uint8_t)((dataBuffer.size >> 8) & 0xFF);

	UartComm::crc = 0xFFFF;

	for (uint16_t i = 0; i < dataBuffer.size; i++)
	{
		uartTxBuffer[i + 3] = dataBuffer.data[i];
		checkSumCalculate(dataBuffer.data[i]);
	}

	uartTxBuffer[dataBuffer.size + 3] = getCheckSumLow(); //CKL
	uartTxBuffer[dataBuffer.size + 4] = getCheckSumHigh(); //CKH

	uint16_t numBytesSend = dataBuffer.size + 5;

	if (verbose)
	{
		for (uint32_t i = 0; i < numBytesSend; i++)
		{
			std::string tempStr = intToHex(uartTxBuffer[i]);
			std::cout << "[" << tempStr << "] ";
			if ((i + 1) % 16 == 0)
			{
				std::cout << "\n";
			}
		}
		std::cout << "\n";
	}

	boost::system::error_code ec;
//#define USE_LAUNCHPADBSL

#if defined (USE_LAUNCHPADBSL)


	for (uint32_t i = 0; i < numBytesSend; i++)
	{
		sendByte(&uartTxBuffer[i]);
	}

#else

	boost::asio::write(port, boost::asio::buffer(uartTxBuffer, numBytesSend), ec);

#endif



	if (ec)
	{
		throw runtime_error("Error from writing occurs...");
	}

	bytesWrite += numBytesSend;
}

/*******************************************************************************
*Class:       UartComm
*Function:    receiveBuffer
*Description: send the init sequence for P432 family
*Parameters:  uint32_t size: number of bytes that scripter receive from BSL
			  bool verbose: set or unset the verbose mode
*Returns:     uint8_t* &bigBuffer[0] pointer to the receive buffer
*******************************************************************************/
void temp()
{
	std::cout << "got here" << std::endl;
}

uint8_t* UartComm::receiveBuffer(uint32_t size, bool verbose)
{
	
	boost::system::error_code ec;
	

#define RX_WITH_TIMEOUT
#if defined(RX_WITH_TIMEOUT) 
	
	port.get_io_service().reset();
	std::this_thread::sleep_for(std::chrono::milliseconds(200));

	//read asynchronously the first 4 bytes
	cleanInitBuffer();
	boost::asio::async_read(port,
							boost::asio::buffer(&initBuffer, 4),//boost::asio::buffer(&bigBuffer, size),
							boost::bind(&UartComm::readComplete,
										this,
										boost::asio::placeholders::error,
										boost::asio::placeholders::bytes_transferred
						               )
		                   );  

	//Setup a deadline time to implement our timeout
	timer.expires_from_now(boost::posix_time::milliseconds(timeout));

	timer.async_wait(boost::bind(&UartComm::timeOut,
						this,
						boost::asio::placeholders::error
						)
		);

	port.get_io_service().run();

	//get the rest of the bytes
	uint16_t numReadBytes = 0;
	if ((initBuffer[0] == 0x00) && (initBuffer[1] = 0x80))
	{
		numReadBytes = ((((uint16_t)initBuffer[3]) << 8) | initBuffer[2]) + 2;
	}

	if (numReadBytes > 2)
	{
		boost::asio::read(port, boost::asio::buffer(bigBuffer), boost::asio::transfer_exactly(numReadBytes), ec);
	}
	
#else
	
	boost::asio::read(port, boost::asio::buffer(bigBuffer), boost::asio::transfer_exactly(size), ec);

#endif	

	for (int32_t i = numReadBytes; i--;)
	{
		bigBuffer[i+4] = bigBuffer[i];
	}

	for (uint32_t i = 4; i--;)
	{
		bigBuffer[i] = initBuffer[i];
	}


	if ((verbose) && (numReadBytes != 2))
	{
		for (uint32_t i = 0; i < (numReadBytes + 4); i++)
		{
			std::string tempStr = intToHex(bigBuffer[i]);
			std::cout << "<" << tempStr << "> ";
			if ((i + 1) % 16 == 0)
			{
				std::cout << "\n";
			}
		}
		std::cout << "\n";
	}

	size = numReadBytes + 4;
	
	if (numReadBytes > 2)
	{
		uint16_t crcBsl = (((uint16_t)bigBuffer[size - 1]) << 8) | bigBuffer[size - 2];
	
		crc = 0xFFFF;
		for (uint16_t i = 4; i < (size - 2); i++)
		{
			checkSumCalculate(bigBuffer[i]);
		}
		if (crcBsl != crc)
		{
			std::cout << "Received data error(CRC not matched)... " << std::endl;
		}
	}

	bytesRead += size;
	return &bigBuffer[0];
}

/*******************************************************************************
*Class:       UartComm
*Function:    close
*Description: close the uart communication
*Parameters:  none
*Returns:     none
*******************************************************************************/
void UartComm::close()
{
	port.close();
}

/*******************************************************************************
*Class:       UartComm
*Function:    changeBaudRate
*Description: change the baud rate value based on CHANGE_BAUD_RATE command
*Parameters:  uint32_t baudRate: new baud rate value
*Returns:     none
*******************************************************************************/
void UartComm::changeBaudRate(uint32_t baudRate)
{
	port.set_option(boost::asio::serial_port::baud_rate(baudRate));
}

/*******************************************************************************
*Class:       UartComm
*Function:    retBytesWritten
*Description: return the number of bytes that sent through uart
*Parameters:  none
*Returns:     uint32_t bytesWrite
*******************************************************************************/
uint32_t UartComm::retBytesWritten()
{
	return bytesWrite;
}

/*******************************************************************************
*Class:       UartComm
*Function:    retBytesRead
*Description: return the number of bytes that received through uart
*Parameters:  none
*Returns:     uint32_t bytesRead
*******************************************************************************/
uint32_t UartComm::retBytesRead()
{
	return bytesRead;
}

/*******************************************************************************
*Class:       UartComm
*Function:    resetBytesWritten
*Description: reset the number of bytes that sent through uart
*Parameters:  none
*Returns:     none
*******************************************************************************/
void UartComm::resetBytesWritten()
{
	bytesWrite=0;
}

/*******************************************************************************
*Class:       UartComm
*Function:    resetBytesRead
*Description: reset the number of bytes that received through uart
*Parameters:  none
*Returns:     none
*******************************************************************************/
void UartComm::resetBytesRead()
{
	bytesRead=0;
}