/*
BSL-Scripter-3.0.1.0

==UsbComm.cpp==

Copyright (c) 2008  2015 Texas Instruments Incorporated

All rights reserved not granted herein.

Limited License.

Texas Instruments Incorporated grants a world-wide, royalty-free,
non-exclusive license under copyrights and patents it now or
hereafter owns or controls to make, have made, use, import,
offer to sell and sell ("Utilize") this software subject to
the terms herein.  With respect to the foregoing patent license,
such license is granted  solely to the extent that any such patent
is necessary to Utilize the software alone.  The patent license
shall not apply to any combinations which include this software,
other than combinations with devices manufactured by or for TI
(TI Devices). No hardware patent is licensed hereunder.

Redistributions must preserve existing copyright notices and
reproduce this license (including the above copyright notice
and the disclaimer and (if applicable) source code license
limitations below) in the documentation and/or other materials
provided with the distribution

Redistribution and use in binary form, without modification,
are permitted provided that the following conditions are met:

*	No reverse engineering, decompilation, or disassembly of
this software is permitted with respect to any software
provided in binary form.

*	any redistribution and use are licensed by TI for use only
with TI Devices.

*	Nothing shall obligate TI to provide you with source code
for the software licensed and provided to you in object code.

If software source code is provided to you, modification and
redistribution of the source code are permitted provided that
the following conditions are met:

*	any redistribution and use of the source code, including any
resulting derivative works, are licensed by TI for use only
with TI Devices.

*	any redistribution and use of any object code compiled from
the source code and any resulting derivative works, are licensed
by TI for use only with TI Devices.

Neither the name of Texas Instruments Incorporated nor the names
of its suppliers may be used to endorse or promote products derived
from this software without specific prior written permission.

DISCLAIMER.

THIS SOFTWARE IS PROVIDED BY TI AND TIS LICENSORS "AS IS" AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL TI AND TIS LICENSORS
BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

Version:	Release:		Author:
3.0.1.0		Aug/07/2015		Fatmawati Santosa

*/
#include "stdafx.h"

//Constructor
UsbComm::UsbComm()
	: handle(nullptr)
{

}

/*******************************************************************************
*Class:       UsbComm
*Function:    init
*Description: setup the uart communication port
*Parameters:  ActualCommandParams actualCommandParams: get the commport
*Returns:     none
*******************************************************************************/
void UsbComm::init(ActualCommandParams actualCommandParams)
{
	if (hid_init())
	{
		throw std::runtime_error("HID init failed");
	}

	UsbComm::handle = hid_open(VID, PID, nullptr);

	if (!handle)
	{
		throw std::runtime_error("Unable to open the HID device");
	}

}

/*******************************************************************************
*Class:       UsbComm
*Function:    transmitBufferInit432
*Description: send the init sequence for P432 family
*Parameters:  none
*Returns:     exception
*******************************************************************************/
void UsbComm::transmitBufferInit432()
{
	throw runtime_error("No USB communication from P432...");
}

/*******************************************************************************
*Class:       UsbComm
*Function:    transmitBuffer
*Description: send data buffer of BSl command
*Parameters:  DataBuffer dataBuffer: buffer contain BSL command to be transmitted
bool verbose: set or unset the verbose mode
*Returns:     none
*******************************************************************************/
void UsbComm::transmitBuffer(DataBuffer dataBuffer, bool verbose)
{
	cleanBigBuffer();

	uint8_t usbSendBuffer[63];

	usbSendBuffer[0] = 0x3F;
	usbSendBuffer[1] = dataBuffer.size;

	for (uint32_t i = 0; i < dataBuffer.size; i++)
	{
		usbSendBuffer[i + 2] = dataBuffer.data[i];
	}

	int32_t res = hid_write(handle, usbSendBuffer, (dataBuffer.size + 2));

	if (res < 0)
	{
		throw runtime_error("Data is not completely transmitted...");
	}

	bytesWrite += dataBuffer.size;
	
	if (verbose)
	{
		for (uint32_t i = 0; i < (dataBuffer.size + 2); i++)
		{
			std::string tempStr = intToHex(usbSendBuffer[i]);
			std::cout << "[" << tempStr << "] ";
			if ((i + 1) % 16 == 0)
			{
				std::cout << "\n";
			}
		}
		std::cout << "\n";
	}
}

/*******************************************************************************
*Class:       UsbComm
*Function:    receiveBuffer
*Description: send the init sequence for P432 family
*Parameters:  uint32_t size: number of bytes that scripter receive from BSL
bool verbose: set or unset the verbose mode
*Returns:     uint8_t* &bigBuffer[0] pointer to the receive buffer
*******************************************************************************/
uint8_t* UsbComm::receiveBuffer(uint32_t size, bool verbose)
{
//#define NEW_USB

#if defined(NEW_USB)
	uint8_t headerTable[2];
	int32_t res = hid_read_timeout(handle, &bigBuffer[0], 2, MS_DELAY_RX);

	if (res < 0)
	{
		std::cout << "Data is not completely received..." << std::endl;
	}

	if (bigBuffer[1] > 0)
	{
		std::cout << (int)headerTable[1] << std::endl;
		res = hid_read_timeout(handle, &bigBuffer[2], (bigBuffer[1] + 1), MS_DELAY_RX);
	}

	if ((verbose) && (bigBuffer[1] != 0))
	{
		for (uint32_t i = 0; i < (bigBuffer[1] + 2); i++)
		{
			std::string tempStr = intToHex(bigBuffer[i]);
			std::cout << "<" << tempStr << "> ";
			if ((i + 1) % 16 == 0)
			{
				std::cout << "\n";
			}
		}
		std::cout << "\n";
	}
	bytesRead += (bigBuffer[1]+2);

#else
 	uint32_t numBytesRx = size;	

	if (size > 1)
	{
		numBytesRx -= 4;

		if (numBytesRx >= 4)
		{
			int32_t res = hid_read_timeout(handle, bigBuffer, numBytesRx, MS_DELAY_RX);

			if (res < 0)
			{
				std::cout << "Data is not completely received..." << std::endl;
				//throw runtime_error("Data is not completely received...");
			}
			if (verbose)
			{
				for (uint32_t i = 0; i < numBytesRx; i++)
				{
					std::string tempStr = intToHex(bigBuffer[i]);
					std::cout << "<" << tempStr << "> ";
					if ((i + 1) % 16 == 0)
					{
						std::cout << "\n";
					}
				}
				std::cout << "\n";
			}
		}
	}
	else
	{
		numBytesRx = 0;
	}
	bytesRead += numBytesRx;
#endif

	
	return &bigBuffer[0];
}

/*******************************************************************************
*Class:       UsbComm
*Function:    close
*Description: close the usb communication
*Parameters:  none
*Returns:     none
*******************************************************************************/
void UsbComm::close()
{
	hid_close(handle);
}

/*******************************************************************************
*Class:       UartComm
*Function:    changeBaudRate
*Description: change the baud rate value based on CHANGE_BAUD_RATE command
*Parameters:  uint32_t baudRate: new baud rate value
*Returns:     exception
*******************************************************************************/
void UsbComm::changeBaudRate(uint32_t baudRate)
{
	throw runtime_error("Change Baud Rate is not applicable for USB communication...");
}

/*******************************************************************************
*Class:       UsbComm
*Function:    retBytesWritten
*Description: return the number of bytes that sent through uart
*Parameters:  none
*Returns:     uint32_t bytesWrite
*******************************************************************************/
uint32_t UsbComm::retBytesWritten()
{
	return bytesWrite;
}

/*******************************************************************************
*Class:       UsbComm
*Function:    retBytesRead
*Description: return the number of bytes that received through uart
*Parameters:  none
*Returns:     uint32_t bytesRead
*******************************************************************************/
uint32_t UsbComm::retBytesRead()
{
	return bytesRead;
}

/*******************************************************************************
*Class:       UsbComm
*Function:    resetBytesWritten
*Description: reset the number of bytes that sent through uart
*Parameters:  none
*Returns:     none
*******************************************************************************/
void UsbComm::resetBytesWritten()
{
	bytesWrite = 0;
}

/*******************************************************************************
*Class:       UsbComm
*Function:    resetBytesRead
*Description: reset the number of bytes that received through uart
*Parameters:  none
*Returns:     none
*******************************************************************************/
void UsbComm::resetBytesRead()
{
	bytesRead = 0;
}

